@@ -33,10 +33,16 @@ if TYPE_CHECKING:
 
 NATS_HDR_LINE = bytearray(b'NATS/1.0')
 NATS_HDR_LINE_SIZE = len(NATS_HDR_LINE)
+_CRLF_ = b'\r\n'
+_CRLF_LEN_ = len(_CRLF_)
 KV_STREAM_TEMPLATE = "KV_{bucket}"
 KV_PRE_TEMPLATE = "$KV.{bucket}."
 Callback = Callable[['Msg'], Awaitable[None]]
 
+# For JetStream the default pending limits are larger.
+DEFAULT_JS_SUB_PENDING_MSGS_LIMIT = 512 * 1024
+DEFAULT_JS_SUB_PENDING_BYTES_LIMIT = 256 * 1024 * 1024
+
 
 class JetStreamContext(JetStreamManager):
     """
@@ -134,6 +140,9 @@ class JetStreamContext(JetStreamManager):
         ordered_consumer: bool = False,
         idle_heartbeat: Optional[float] = None,
         flow_control: bool = False,
+        pending_msgs_limit: Optional[int] = DEFAULT_JS_SUB_PENDING_MSGS_LIMIT,
+        pending_bytes_limit: Optional[int
+                                      ] = DEFAULT_JS_SUB_PENDING_BYTES_LIMIT,
     ) -> Subscription:
         """Create consumer if needed and push-subscribe to it.
 
@@ -287,6 +296,8 @@ class JetStreamContext(JetStreamManager):
             manual_ack=manual_ack,
             ordered_consumer=ordered_consumer,
             consumer=consumer,
+            pending_msgs_limit=pending_msgs_limit,
+            pending_bytes_limit=pending_bytes_limit,
         )
 
     async def subscribe_bind(
@@ -297,6 +308,9 @@ class JetStreamContext(JetStreamManager):
         cb: Optional[Callback] = None,
         manual_ack: bool = False,
         ordered_consumer: bool = False,
+        pending_msgs_limit: Optional[int] = DEFAULT_JS_SUB_PENDING_MSGS_LIMIT,
+        pending_bytes_limit: Optional[int
+                                      ] = DEFAULT_JS_SUB_PENDING_BYTES_LIMIT,
     ) -> Subscription:
         """Push-subscribe to an existing consumer.
         """
@@ -310,6 +324,8 @@ class JetStreamContext(JetStreamManager):
             subject=config.deliver_subject,
             queue=config.deliver_group or "",
             cb=cb,
+            pending_msgs_limit=pending_msgs_limit,
+            pending_bytes_limit=pending_bytes_limit,
         )
         psub = JetStreamContext.PushSubscription(self, sub, stream, consumer)
 
@@ -343,6 +359,9 @@ class JetStreamContext(JetStreamManager):
         durable: str,
         stream: Optional[str] = None,
         config: Optional[api.ConsumerConfig] = None,
+        pending_msgs_limit: Optional[int] = DEFAULT_JS_SUB_PENDING_MSGS_LIMIT,
+        pending_bytes_limit: Optional[int
+                                      ] = DEFAULT_JS_SUB_PENDING_BYTES_LIMIT,
     ) -> "JetStreamContext.PullSubscription":
         """Create consumer and pull subscription.
 
@@ -387,13 +406,21 @@ class JetStreamContext(JetStreamManager):
             config.durable_name = durable
             await self._jsm.add_consumer(stream, config=config)
 
-        return await self.pull_subscribe_bind(durable=durable, stream=stream)
+        return await self.pull_subscribe_bind(
+            durable=durable,
+            stream=stream,
+            pending_bytes_limit=pending_bytes_limit,
+            pending_msgs_limit=pending_msgs_limit,
+        )
 
     async def pull_subscribe_bind(
         self,
         durable: str,
         stream: str,
         inbox_prefix: bytes = api.INBOX_PREFIX,
+        pending_msgs_limit: Optional[int] = DEFAULT_JS_SUB_PENDING_MSGS_LIMIT,
+        pending_bytes_limit: Optional[int
+                                      ] = DEFAULT_JS_SUB_PENDING_BYTES_LIMIT,
     ) -> "JetStreamContext.PullSubscription":
         """
         pull_subscribe returns a `PullSubscription` that can be delivered messages
@@ -422,7 +449,11 @@ class JetStreamContext(JetStreamManager):
 
         """
         deliver = inbox_prefix + self._nc._nuid.next()
-        sub = await self._nc.subscribe(deliver.decode())
+        sub = await self._nc.subscribe(
+            deliver.decode(),
+            pending_msgs_limit=pending_msgs_limit,
+            pending_bytes_limit=pending_bytes_limit
+        )
         return JetStreamContext.PullSubscription(
             js=self,
             sub=sub,
@@ -649,6 +680,22 @@ class JetStreamContext(JetStreamManager):
             self._nms = f'{prefix}.CONSUMER.MSG.NEXT.{stream}.{consumer}'
             self._deliver = deliver.decode()
 
+        @property
+        def pending_msgs(self) -> int:
+            """
+            Number of delivered messages by the NATS Server that are being buffered
+            in the pending queue.
+            """
+            return self._sub._pending_queue.qsize()
+
+        @property
+        def pending_bytes(self) -> int:
+            """
+            Size of data sent by the NATS Server that is being buffered
+            in the pending queue.
+            """
+            return self._sub._pending_size
+
         async def unsubscribe(self) -> None:
             """
             unsubscribe destroys de inboxes of the pull subscription making it
@@ -727,6 +774,7 @@ class JetStreamContext(JetStreamManager):
             while not queue.empty():
                 try:
                     msg = queue.get_nowait()
+                    self._sub._pending_size -= len(msg.data)
                     status = JetStreamContext.is_status_msg(msg)
                     if status:
                         # Discard status messages at this point since were meant
@@ -781,6 +829,7 @@ class JetStreamContext(JetStreamManager):
             while not queue.empty():
                 try:
                     msg = queue.get_nowait()
+                    self._sub._pending_size -= len(msg.data)
                     status = JetStreamContext.is_status_msg(msg)
                     if status:
                         # Discard status messages at this point since were meant
@@ -922,11 +971,13 @@ class JetStreamContext(JetStreamManager):
         req_subject = f"{self._prefix}.STREAM.MSG.GET.{stream_name}"
         req = {'last_by_subj': subject}
         data = json.dumps(req)
-        resp = await self._api_request(req_subject, data.encode())
+        resp = await self._api_request(
+            req_subject, data.encode(), timeout=self._timeout
+        )
         raw_msg = api.RawStreamMsg.from_response(resp['message'])
         if raw_msg.hdrs:
             hdrs = base64.b64decode(raw_msg.hdrs)
-            raw_headers = hdrs[len(NATS_HDR_LINE):]
+            raw_headers = hdrs[NATS_HDR_LINE_SIZE + _CRLF_LEN_:]
             parsed_headers = self._jsm._hdr_parser.parsebytes(raw_headers)
             headers = None
             if len(parsed_headers.items()) > 0:
